<?php
namespace Financeiro\Logic\ContasPagar\Cnab\Retorno;

use \Financeiro\Entities\Errcnabp;
use Geral\Entities\Banco;
use Geral\Util\Ambiente;
use \Fiscal\Entities\Boletpagar;
use \Indeterminado\Entities\Antecipaforne;
use \Financeiro\Entities\GeralPag;
use \Financeiro\Entities\Ajbolpag;
use \Geral\Entities\Contcorr;



/**
 * Created by JetBrains PhpStorm.
 * User: alanoliveira
 * Date: 28/04/14
 * Time: 14:11
 * To change this template use File | Settings | File Templates.
 */


abstract class LeitorCnab {

    const CODIGO_COMPENSACAO_SANTANDER = "033";
    CONST CODIGO_COMPENSACAO_BRADESCO  = "237";
    const CODIGO_COMPENSACAO_ITAU = "341";

    protected $erros = array();
    protected $ocorrencias = array();
    protected $contlinha = 0;

    protected $cdempresa;

    protected $cdbanco;

    protected $arquivo;
    // Armazaneza objetoa do tipo \Retorno.
    protected $retornos = array();

    /** @var $retorno Retorno*/
    protected $retorno;

    /**
     * @return array
     */
    public function getRetornos(){
        return $this->retornos;
    }

    /**
     * @return array
     */
    public function getErros(){
        return $this->erros;

    }
    /**
     * @param $cdempresa
     * @param $cdbanco
     * @param $arquiuvo
     * @throws \Exception
     * @return LeitorCnabSantander
     */
    public static function factory($cdempresa,$cdbanco, $arquiuvo)
    {
        try
        {
            if ($cdbanco === self::CODIGO_COMPENSACAO_SANTANDER)
                return new LeitorCnabSantander($arquiuvo, $cdempresa, $cdbanco);

            elseif($cdbanco == self::CODIGO_COMPENSACAO_BRADESCO)
                return new LeitorCnabBradesco($arquiuvo, $cdempresa, $cdbanco);

            elseif($cdbanco == self::CODIGO_COMPENSACAO_ITAU)
                return new LeitorCnabItau($arquiuvo, $cdempresa, $cdbanco);

            else throw new \Exception('O Códido de Compensação informado não pertece a nenhum banco reconhecido.');

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public abstract function leArquivo();

    /**
     * @param $cdlinha
     * @param $erro
     * @param bool $iderroGeral
     * @internal param $errom
     * @internal param $erro
     * @internal param $linha
     */
    protected function gravaErros($cdlinha, $erro,$iderroGeral = false){
        $this->erros[] = array('CDLINHA'=>str_pad($cdlinha,10,'0',STR_PAD_LEFT),
                               'DSERRO' =>$erro,
                               'IDGERAL' => $iderroGeral);
    }

    /**
     * @param $tipomovimento
     */
    protected function tipoMovimento($tipomovimento){

        $tipomovimento = trim($tipomovimento);
        if(empty($tipomovimento) and $tipomovimento != 0){
            self::gravaErros($this->contlinha,'Tipo de Movimento não Informado.');
        }
        $this->retorno->setTpmovimento($tipomovimento);

    }


    /**
     * @param $cdbanco
     */
    public function preparaOcorrencias($cdbanco){
        /** @var $retorno Retorno */
        foreach($this->retornos as $retorno){

            $ocorrencia = $retorno->getOcorrencia();
            $tamOcorrencia = strlen($retorno->getOcorrencia());

            for($i = 0;$i<$tamOcorrencia;$i+=2){
                $cderro = substr($ocorrencia,$i,2);
                if($cderro !== "  "){

                    /** @var $errcnabp Errcnabp */
                    $errcnabp = Errcnabp::findOneByCdbancoAndCderrcnabpAndNrorg($cdbanco, $cderro, Ambiente::getNrorg());

                    $dserro = $cderro;
                    if($errcnabp)
                        $dserro.= " - {$errcnabp->getDserrcnabp()}";

                        $this->erros[] = array('CDLINHA'=>$retorno->getCdlinha(),
                                               'CDERRO' =>$cderro,
                                               'DSERRO' =>$dserro,
                                               'IDGERAL' => false);
                }
            }
        }
    }

    /** Busca Tipo de Baixa para Pagamento no Boleto ou na Antecipação
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $nrorg
     * @return null
     */
    protected function getTipoBaixaPagamento($cdfilial, $nrlanbolpag, $nrorg){

        /** @var $boletpagar Boletpagar */
        $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($cdfilial, $nrlanbolpag, $nrorg);

        if($boletpagar and $boletpagar->getCdtpbaixapag()){
            return $boletpagar->getCdtpbaixapag();
        }
        else{
            /** @var $antecipaforne Antecipaforne */
            $antecipaforne = Antecipaforne::findOneByNrlancantecipaAndCdfilialAndNrorg($cdfilial, $nrlanbolpag, $nrorg);
            if($antecipaforne and $antecipaforne->getCdtpbaixapag())
                return $antecipaforne->getCdtpbaixapag();
            else return null;
        }
    }

    /**
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $nrorg
     * @return null
     */
    protected function getReducont($cdfilial, $nrlanbolpag, $nrorg){
        /** @var $boletpagar Boletpagar */
        $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($cdfilial, $nrlanbolpag, $nrorg);

        if($boletpagar and $boletpagar->getCdreducont()){
            return $boletpagar->getCdreducont();
        }
        else{
            /** @var $antecipaforne Antecipaforne */
            $antecipaforne = Antecipaforne::findOneByNrlancantecipaAndCdfilialAndNrorg($cdfilial, $nrlanbolpag, $nrorg);
            if($antecipaforne and $antecipaforne->getCdreducontpag())
                return $antecipaforne->getCdreducontpag();
            else return null;
        }
    }

    /**
     * @return mixed
     */
    private function getBanco(){

        $banco = Banco::findOneByCdbancoAndNrorg($this->cdbanco, \Ambiente::getNrorg());
        return $banco;

    }

    public function validaBoletos(){

        $contasVerificadas = array();
        /** @var $retorno Retorno  */
        foreach($this->retornos as &$retorno){

            //Se for um boleto
            if($retorno->getBoleto()){

                $retorno->setSituacaoOk(true);

                if(!$retorno->getCdfilial() or !$retorno->getNrlanbolpag()){
                    self::gravaErros($retorno->getCdlinha(),'Boleto não Cadastrado');
                    $retorno->setSituacaoOk(false);
                }
                else{
                    /** @var $boletpagar Boletpagar */
                    $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($retorno->getCdfilial(),
                                                                                      $retorno->getNrlanbolpag(),
                                                                                      \Ambiente::getNrorg());
                    if(! $boletpagar){
                        self::gravaErros($retorno->getCdlinha(),'Boleto não Cadastrado.');
                        $retorno->setSituacaoOk(false);
                    }
                    else{

                        if($boletpagar->getNrbaixapag()){
                            self::gravaErros($retorno->getCdlinha(),'Boleto encontra-se baixado.');
                            $retorno->setSituacaoOk(false);
                        }

                        if($this->cdempresa != $boletpagar->getCdempresa()){
                            self::gravaErros($retorno->getCdlinha(), 'Empresa do Boleto é Diferenta da Empresa Informada.');
                            $retorno->setSituacaoOk(false);
                        }


                        if(round($retorno->getVracrescimo(),2)>0){
                            /** @var $geralpag GeralPag */
                            $geralpag = GeralPag::findOneByNrorg(Ambiente::getNrorg());
                            if(! $geralpag->getCdajacrepag()){
                                self::gravaErros($retorno->getCdlinha(),
                                                'Tipo de ajuste não está parametrizado.');
                                                $retorno->setSituacaoOk(false);
                            }
                            else{
                            $ajbolpag = Ajbolpag::findOneByCdfilialAndNrlanbolpagAndCdajuctpagAndNrorg
                                                  ($retorno->getCdfilial(),
                                                  $retorno->getNrlanbolpag(),
                                                  $geralpag->getCdajacrepag(),
                                                  \Ambiente::getNrorg());

                                if($ajbolpag){
                                    self::gravaErros($retorno->getCdlinha(),'Boleto já possui ajuste cadastrado para o
                                                                             tipo de ajuste parametrizado');
                                    $retorno->setSituacaoOk(false);
                                }
                            }
                        }

                        if(round($retorno->getVrdesconto(),2)>0){
                            /** @var $geralpag GeralPag */
                            $geralpag = GeralPag::findOneByNrorg(Ambiente::getNrorg());
                            if(! $geralpag->getCdajdescpag()){
                                self::gravaErros($retorno->getCdlinha(),
                                                 'Tipo de ajuste não está parametrizado .');
                                                  $retorno->setSituacaoOk(false);
                            }
                            else{
                                $ajbolpag = Ajbolpag::findOneByCdfilialAndNrlanbolpagAndCdajuctpagAndNrorg
                                                      ($retorno->getCdfilial(),
                                                      $retorno->getNrlanbolpag(),
                                                      $geralpag->getCdajdescpag(),
                                                      \Ambiente::getNrorg());

                                if($ajbolpag){
                                    self::gravaErros($retorno->getCdlinha(),'Boleto já possui ajuste cadastrado para o
                                                                             tipo de ajuste parametrizado');
                                    $retorno->setSituacaoOk(false);
                                }
                            }
                        }

                        if(! $boletpagar->getCdreducont()){
                            self::gravaErros($retorno->getCdlinha(),'Boleto não possui conta corrente cadastrada.');
                            $retorno->setSituacaoOk(false);
                        }

                        else{/*
                            if($retorno->getCdagencia() and $retorno->getCdconta()){

                                // Verifica se a conta e agencia do boleto ja foram verificadas
                                $key = array_search(array('CDBANCO' => $this->cdbanco,
                                                          'CDAGENCIA' => $retorno->getCdagencia(),
                                                          'CDCONTCORR' => $retorno->getCdconta(),
                                                          'CDEMPRESA' => $this->cdempresa,
                                                          'CDREDUCONT' => $retorno->getCdreducont()
                                                          )
                                                   ,$contasVerificadas);

                                if (!$key)
                                {

                                    $banco = self::getBanco();

                                    $lenPictCont = strlen($banco->getCdpictcont());

                                    if(strpos($banco->getCdpictcont(),'-') !== false)
                                        $lenPictCont = strpos($banco->getCdpictcont(),'-');


                                    $repositorio = Contcorr::getRepository();
                                    $result = $repositorio->getContCorrRetornoCnab($this->cdempresa,
                                                                                   $this->cdbanco,
                                                                                   $retorno->getCdagencia(),
                                                                                   $retorno->getCdconta(),
                                                                                   \Ambiente::getNrorg(),
                                                                                   $lenPictCont);
                                    if($result){
                                        $result = $result[0];
                                        $contasVerificadas[] = array($result['CDBANCO'],
                                                                     $result['CDAGENCIA'],
                                                                     $result['CDCONTCORR'],
                                                                     $result['CDEMPRESA'],
                                                                     $result['CDREDUCONT']);

                                        $key = count($contasVerificadas)-1;
                                    }


                                }

                                if($key == null)
                                    $conta = null;
                                else
                                    $conta = $contasVerificadas[$key];
                                if(! $conta or $boletpagar->getCdreducont() != $conta['CDREUCONT']){

                                    if((! $conta) or ($conta['CDBANCO'] != $boletpagar->getReducont()->getCdbanco() or
                                       $conta['CDEMPRESA'] != $boletpagar->getCdempresa())){

                                        self::gravaErros($retorno->getCdlinha(),
                                                         "Conta corrente de débito {$this->cdbanco} /
                                                         {$retorno->getCdagencia()} / {$retorno->getCdconta()}



                                        é diferente da conta corrente do boleto.");
                                        $retorno->setSituacaoOk(false);
                                    }
                                }
                            }
                            else{
                                if($boletpagar->getReducont()->getCdbanco() != $this->cdbanco or
                                   $boletpagar->getCdempresa() != $this->cdempresa){

                                    self::gravaErros($retorno->getCdlinha(),
                                                     "Conta corrente de débito {$this->cdbanco} /
                                                     {$retorno->getCdagencia()} / {$retorno->getCdconta()}
                                                     é diferente da conta corrente do boleto.");
                                    $retorno->setSituacaoOk(false);
                                }
                            }*/
                        }

                        if(! $boletpagar->getCdtpbaixapag()){
                            self::gravaErros($retorno->getCdlinha(),'Boleto não possui tipo de baixa a pagar cadastrado');
                            $retorno->setSituacaoOk(false);
                        }

                        if(! $boletpagar->getNrcnabpagrem()){
                            self::gravaErros($retorno->getCdlinha(),'Boleto não está ligado a uma remessa.');
                            $retorno->setSituacaoOk(false);
                        }
                    }
                }
            }
        }
    }
}